#include "counterwidget.h"
#include "commonstatic.h"

CounterWidget::CounterWidget(QWidget *parent, Counter* counter) :
    QScrollArea(parent)
{
    m_Counter = counter;

    setFrameStyle(QFrame::NoFrame | QFrame::Plain);
    setWidgetResizable(true);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);

    m_Container = new QWidget(this);
    m_Container->setStyleSheet("color: black;");
    m_Container->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Maximum);
    setWidget(m_Container);

    m_Layout = new QGridLayout( m_Container );
    m_Layout->setSpacing(3);
    m_Layout->setMargin( 0 );
    m_Container->setLayout(m_Layout);

    CreateWidgets();
}

CounterWidget::~CounterWidget()
{
    delete m_Layout;
    delete m_Container;
}

void CounterWidget::CreateWidgets()
{
    int row = 0;
    QLabel* lbl = new QLabel(this);
    lbl->setStyleSheet("color: red;");
    lbl->setText(m_Counter->Description);
    m_Layout->addWidget(lbl, row, 0, 1, 2);

    row++;
    QProgressBar* bar = new QProgressBar(this);
    bar->setMaximumHeight(30);
    bar->setMaximum(m_Counter->Soglia);
    bar->setValue(m_Counter->Utilizzato);
    bar->setTextVisible(false);
    bar->setStyleSheet("QProgressBar:horizontal { \
                        border: 1px solid gray; \
                        border-radius: 3px; \
                        background: white; \
                        padding: 1px; \
                        } \
                        QProgressBar::chunk:horizontal { \
                        background: qlineargradient(x1: 0, y1: 0.5, x2: 1, y2: 0.5, stop: 0 red, stop: 1 white); \
                        }");
    m_Layout->addWidget(bar, row, 0, 1, 2);

    row++;
    lbl = new QLabel(this);
    lbl->setText(QString(tr("Scatti utilizzati: %1 / %2")).arg(m_Counter->Utilizzato).arg(m_Counter->Soglia));
    m_Layout->addWidget(lbl, row, 0, 1, 1);

    if (m_Counter->DimensioneScatto == "Secondi"){
        lbl = new QLabel(this);
        lbl->setAlignment(Qt::AlignRight);
        lbl->setText(QString(tr("Ore: %1 / %2")).arg(ToHourMinutes(m_Counter->Utilizzato * m_Counter->Scatto)).arg(ToHourMinutes(m_Counter->Soglia * m_Counter->Scatto)));

        if (CommonStatic::IsPortrait()){
            lbl->setAlignment(Qt::AlignLeft);
            m_Layout->addWidget(lbl, ++row, 0, 1, 1);
        }else
            m_Layout->addWidget(lbl, row, 1, 1, 1);
    }

    row++;
    lbl = new QLabel(this);
    lbl->setText(QString(tr("Validit: %1 - %2")).arg(m_Counter->Periodo.toString("dd/MM/yyyy")).arg(m_Counter->DataUtile.toString("dd/MM/yyyy")));
    m_Layout->addWidget(lbl, row, 0, 1, 1);
}

QString CounterWidget::ToHourMinutes(int seconds)
{
    int hh = seconds / 3600;
    int mm = (seconds - (hh * 3600)) / 60;
    QString s;
    return QString("%1:%2").arg(hh, 2, 10, QChar('0')).arg(mm, 2, 10, QChar('0'));
}
